import { Component, OnInit, AfterContentInit } from '@angular/core';
import { MatCheckboxChange } from '@angular/material';
import { Store, select } from '@ngrx/store';
import { RPCError } from 'packages/core/rpc/error';
import {
  DiscoveryStartInfo,
  DiscoveryZone,
  Zone
} from '../../model';
import * as CIDR from 'ip-cidr';
import * as DiscoveredStore from '../../store/setting';
import * as DiscoverStore from '../../store/notification';

import { SettingSelector, NotificationSelector } from '../../store';

@Component({
  selector: 'of-setting',
  templateUrl: './setting.component.html',
  styleUrls: ['./setting.component.scss']
})
export class SettingComponent implements OnInit, AfterContentInit {

  settingSucceed$ = this.discoverdstore.pipe(select(SettingSelector.select('isStart')));
  discoveryResult$ = this.discoverstore.pipe(select(NotificationSelector.select('getDiscoveryResult')));

  started = false;

  cidr;
  startIP;
  endIP;
  startPort;
  endPort;
  // private settingForm: FormGroup;
  hostChecked = true;
  portChecked = true;
  serviceChecked = true;

  serviceItems = [
    { name: 'MySQL' },
    { name: 'Redis' },
    { name: 'Tomcat' },
    { name: 'Nginx' },
  ];

  hosts = nodes;

  checkedSet = new Set();

  constructor(
    private discoverdstore: Store<DiscoveredStore.State>,
    private discoverstore: Store<DiscoverStore.State>,
  ) {
  }

  ngOnInit() {
    this.settingSucceed$.subscribe(
      (succeed: boolean) => {
        if (succeed) {
          this.started = true;
        }
      },
      (error: RPCError) => {
        console.log(error.response.message);
      }
    );

    this.discoveryResult$.subscribe(
      (zone: Zone) => {
        this.convertTreeView(zone);
      },
      (error: RPCError) => {
        console.log(error.response.message);
      }
    );


  }

  ngAfterContentInit() {
    this.cidr = '192.168.1.0/24';
    this.hostRange();
    this.portRange();
  }

  hostRange() {
    const cidr = new CIDR(this.cidr);
    if (!cidr.isValid()) {
      alert('Invalid cidr : ' + this.cidr);
      return;
    }
    this.startIP = cidr.addressStart.address;
    this.endIP = cidr.addressEnd.address;
  }

  portRange() {
    this.startPort = '1';
    this.endPort = '1024';
  }

  handleHostCheckChange(e: MatCheckboxChange) {
    this.hostChecked = true;
    e.source.checked = true;
  }

  handlePortCheckChange(e: MatCheckboxChange) {
    this.serviceChecked = false;
  }

  handleServiceCheckChange(e: MatCheckboxChange) {
  }

  discovery() {
    if (!this.portChecked) {
      this.startPort = '0';
      this.endPort = '0';
    }
    if (!this.serviceChecked) {

    }
    const startInfo: DiscoveryStartInfo = {
      startIp: this.startIP,
      endIP: this.endIP,
      startPort: this.startPort,
      endPort: this.endPort
    };

    const discoveryZone: DiscoveryZone = {
      discoveryHost: {
        firstScanRange: this.startIP,
        lastScanRange: this.endIP,
      }
    };

    // this.store.dispatch(new DiscoverySettingStore.Setting(startInfo));
    this.discoverstore.dispatch(new DiscoverStore.DiscoverZone(
      {probeID: '52abd6fd57e511e7ac52080027658d13', discoveryZone: discoveryZone}));
    this.started = true;
  }

  checkDiscoveryResult(node) {

    node.toggleActivated(true);

    if (node.isActive) {
      if (this.checkedSet.has(node.data) === false) {
        this.checkedSet.add(node.data);
      }
    } else {
      if (this.checkedSet.has(node.data)) {
        this.checkedSet.delete(node.data);
      }
    }

  }

  discoveryResultSave() {
    console.log(this.checkedSet);
  }

  convertTreeView(zone: Zone) {

    if (zone === undefined || zone === null) {
      return;
    }

    const treeNodes = this.convertViewHost(zone.hosts);

    console.log(treeNodes);
  }

  convertViewHost(hosts): any[] {

    const treeNodes: any[] = [];

    hosts.forEach((host, hostKey) => {

      const jHost: any = {
        title: 'Host - ' + host.ip,
        className : 'cn' + host.ip
      };
      jHost.obj = host;

      jHost.children = this.convertViewPort(host.ports);

      treeNodes.push(jHost);

    });

    return treeNodes;

  }

  convertViewPort(ports): any[] {

    if (ports === undefined && ports.size <= 0) {
      return null;
    }

    const portChildren: any[] = [];

    ports.forEach((port, portKey) => {

      const jPort: any = {
        title: 'Port - ' + port.portNumber,
        className : 'cn' + port.portNumber,
      };
      jPort.obj = port;

      jPort.children = this.convertViewService(port.services);

      portChildren.push(jPort);
    } );


    return portChildren;
  }

  convertViewService(services): any[] {

    if (services === undefined && services.size <= 0) {
      return null;
    }

    const serviceChildren: any[] = [];

    services.forEach((service, serviceKey) => {
      const jService: any = {
        title: 'Service - ' + service.serviceName,
        className : 'cn' + service.serviceName,
      };
      jService.obj = service;

      serviceChildren.push(jService);
    });

    return serviceChildren;
  }

}




const nodes = [
  {
    title: 'host - 3232235781',
    className: 'className3232235781',
    children: [
      {
        title: 'Port - 22',
        className: 'className22',
        children: [
          {
            title: 'SSH',
            className: 'classNameSSH'
          }
        ]
      },
      {
        title: 'Port - 80',
        className: 'className80',
        children: [
          {
            title: 'HTTP',
            className: 'classNameHTTP'
          }
        ]
      },
      {
        title: 'Port - 1936',
        className: 'className1936',
        children: [
          {
            title: 'HTTP',
            className: 'classNameHTTP'
          }
        ]
      }
    ]
  },
  {
    title: 'host - 3232235781',
    className: 'className3232235781',
    children: [
      {
        title: 'Port - 22',
        className: 'className22',
        children: [
          {
            title: 'SSH',
            className: 'classNameSSH'
          }
        ]
      },
      {
        title: 'Port - 80',
        className: 'className80',
        children: [
          {
            title: 'HTTP',
            className: 'classNameHTTP'
          }
        ]
      },
      {
        title: 'Port - 1936',
        className: 'className1936',
        children: [
          {
            title: 'HTTP',
            className: 'classNameHTTP'
          }
        ]
      }
    ]
  },
  {
    title: 'host - 3232235781',
    className: 'className3232235781',
    children: [
      {
        title: 'Port - 22',
        className: 'className22',
        children: [
          {
            title: 'SSH',
            className: 'classNameSSH'
          }
        ]
      },
      {
        title: 'Port - 80',
        className: 'className80',
        children: [
          {
            title: 'HTTP',
            className: 'classNameHTTP'
          }
        ]
      },
      {
        title: 'Port - 1936',
        className: 'className1936',
        children: [
          {
            title: 'HTTP',
            className: 'classNameHTTP'
          }
        ]
      }
    ]
  }
];

